#ifndef SCRAWLERDATA
#define SCRAWLERDATA

#endif // SCRAWLERDATA

#include <QString>
#include <QStringList>

class SCrawlerData
{
public:
    enum E_COLUMN
    {
        PLATFORM_NAME = 0,
        PLATFORM_FORM,
        PLATFORM_TITLE,
        ARTICLE_FORM,
        ARTICLE_PARENT,
        ARTICLE_ID,
        ARTICLE_NICKNAME,
        ARTICLE_TITLE,
        ARTICLE_DATA,
        ARTICLE_URL,
        ARTICLE_HIT,
        ARTICLE_DATE,
        ARTICLE_ORDER,
        ARTICLE_PROFILE,
        ARTICLE_PROFILEURL,
        PLATFORM_ID,
        KEYWORD_ID,
        REPLY_URL,
        ETC,
        TOTAL_COUNT,
    };

private:
    QString m_strData[TOTAL_COUNT];
    QString m_strColumn[TOTAL_COUNT];
    QString m_strTable;

public:
    SCrawlerData();
    ~SCrawlerData();
    QString getData(int _num);
    QString SqlString(QString _str);
    QString GetSafeUtf(QString _strData);
    QString GetTable();
    QString GetDate(QString _strDate);
    void setData(QString _str, int _num);
    void clear();
    void clear(int _num);
    bool sendDB();
    bool deleteDB(QString _str, int _num);
    void setTable(QString _str);
};
